require "refile/nobrainer"

module Nukleus
    class Document
        include NoBrainer::Document
        include NoBrainer::Document::Timestamps
        extend Refile::NoBrainer::Attachment

        after_destroy :remove_all_versions

        # A document can exist in many versions. Each has its own set of meta data and its own document.
        field       :document_id,                 :type => String, :required => true, :index => true
        field       :document_version,            :type => Integer, :required => true, :default => 0, :validates => {:uniqueness => {:scope => :document_id}}

        # The attached file
        attachment  :document_file
        field       :document_file_filename,      :type => String
        field       :document_file_size,          :type => Integer
        field       :document_file_content_type,  :type => String

        # Meta-Data
        field       :tags,                        :type => Array
        field       :meta_data,                   :type => Hash

        # Update with creating a copy with a higher version number.
        # @return returns the newly created and modified copy of the document or the current document on error.
        def update_with_copy(params)
            success = true
            if success && params.include?(:document_id)
                # Remove document_id from params in order to avoid unexpected duplications.
                # This should be an error. But if we expect not to have this param, we will get a problem
                # in DocumentController::document_params  due to an empty Document-Hash.
                params.delete(:document_id)
                success = true
            end
            if success && params.include?(:document_version)
                errors.add(:document_version, "document_version cannot be changed")
                success = false
            end

            if success
                # make a deep copy of the current document and update data with this new version
                new_document_hash = self.raw_attributes
                new_document_hash.delete(:id)
                new_document_hash[:document_version] += 1
                new_document = Document.new(new_document_hash)
                success = new_document.update(params)
            end

            # The original implementation returns true or false!
            success ? new_document : self
        end

        private
        def remove_all_versions
            # Remove all documents with this document_id
            Document.where(document_id: self.document_id).delete_all
            true
        end
    end
end
